using System;
using System.ComponentModel;
using System.Threading;
using System.Net.Sockets;
using System.IO;

public class ATMServer {

  /// <summary>
  /// The ATMServer program          Bishop and Horspool   Feb 2003
  /// =====================   
  /// Simulates the beginning of an ATM protocol.
  /// Allows successive connections.
  /// Illustrates sockets and networking.
  /// </summary>

  Socket socket;
  Thread client;
  static string magicPIN = "5678";
  static int port = 8190;
  static bool done = false;
  Views.Form view;
  ATMServer(int port) {
    view = new Views.Form(
      @"<Form Text='Server'>
          <vertical>
            <Label Text='***** SAVANNA BANK *****' halign=center
                     Font=bold12 />
            <Listbox Name='Output' Width='300' Height='200'
                     Font=sans10 BackColor=BlanchedAlmond/>
            <Button Name='Terminate' Width='110' Font=bold10/>
          </vertical>
        </Form>");
    view.PutText("Output", "Simulating an ATM session on port " +
        port); 
    view.PutText("Output", "");
    
    // Set the server going independently from the Views GUI
    new Thread(new ThreadStart(RunServer)).Start();

    // Keep the GUI going
    GUICommunication();
  }

  public void RunServer() {
    try {
      TcpListener listener = new TcpListener(port);
      listener.Start();
      view.PutText("Output", "Waiting for connection ...");
      int counter = 0;            
        
      while (!done) {            
        // This is where the server sits and waits for clients
        socket = listener.AcceptSocket(); 
        counter++;            
        view.PutText("Output", "Starting a new client, numbered " +
            counter);

        // Start a new thread for a client
        Handler h = new Handler(socket, counter, view);
        client = new Thread(new ThreadStart(h.Run));
        client.Start();
      }
    } catch(Exception) {
      Console.WriteLine("Port " + port +
          " may be busy. Try another.");
    }
  }

  void GUICommunication() {
    while (view != null) {                 
      string b = view.GetControl();
      // Must be terminate
      done = true;
      if (socket != null) {
        socket.Close();    
        client.Join();
      }
      view.PutText("Output", "Server terminated.");
      view.CloseGUI();
    }
  }

  public class Handler {
    Socket socket; 
    NetworkStream socketStream;
    BinaryWriter writer;      
    StreamReader reader;
    Views.Form view;
    int id;
    
    public Handler(Socket s, int i, Views.Form f) {
      socket = s;
      id = i;
      view = f;
    }

    public void Run() {
      try {
        // establish the communication streams
        socketStream = new NetworkStream(socket);
        writer = new BinaryWriter(socketStream);           
        reader = new StreamReader(socketStream);

        writer.Write("Connection successful.");
        writer.Write("");

        bool okay = false;
        for (int tries = 0; tries < 3 && !okay; tries++) {
          writer.Write("Please type in your PIN number " +
             "or type CANCEL");
          string message = reader.ReadLine();
          message = message.Substring(1,message.Length-1);
          view.PutText("Output", "Client " + id + ":" + message);
          okay = true;
          switch (message) {
            case "SHUTDOWN" :
              writer.Write("Shutting down. Goodbye.");
              done = true;
              break;
            case "CANCEL" :
              writer.Write("Transaction halted. Goodbye.");
              break;
            default :
              if (message.Equals(magicPIN)) {
                writer.Write("Please start your transactions.");
                break; 
              } else {
                writer.Write("Incorrect PIN. Try again.");
                okay = false;
              }
              break;
          }
        }
        writer.Write("Simulation complete. Thanks.");
        view.PutText("Output", "Simulation complete. Thanks.");
      } catch(Exception error) {
        Console.WriteLine(error.ToString());
      }
    }
  }
  static void Main(string[] args) {
    if (args.Length > 0)
      port = int.Parse(args[0]);
    new ATMServer(port);
  }
}
