using System;
using System.Drawing;
using System.Threading;
using System.Windows.Forms;

public class SpottyThreads : Form {

  ///<summary>
  /// The Spotty Program with threads   Bishop and Horspool  Feb 2003
  /// ===============================
  /// Draws spots of different colours.
  /// Illustrates arrays of simple threads.
  ///</summary>

  static Random r;

  SpottyThreads() {
    //Set up attributes of the graphics form
    this.Width = 450; this.Height = 450;
    this.Text = "Spotty Threads";
    r = new Random();

    // creates three spots objects
    Spots one = new Spots(this, Color.Red,100,20);
    Spots two = new Spots(this, Color.Blue,1000,100);
    Spots three = new Spots(this, Color.Green,10000,200);

    // registers them to threads and starts the threads
    (thread[0] = new Thread(new ThreadStart(one.Run))).Start();
    (thread[1] = new Thread(new ThreadStart(two.Run))).Start();
    (thread[2] = new Thread(new ThreadStart(three.Run))).Start();
  }

  Thread [] thread = new Thread[3]; // Array to hold the threads.
  static bool ended = false;

  protected override void Dispose(bool disposing) {
    ended = true;
    Console.WriteLine("Please wait while we shut down ...");
    for (int i = 0; i < 3; i ++ ) 
      // Pause running this thread until the other threads terminate
      thread[i].Join(); 
    base.Dispose(disposing);
  }

  class Spots {
    Form f;
    Color colour;
    int speed;
    int size;
    SolidBrush brush;
    Graphics g;

    public Spots(Form f, Color c, int s, int z) {
      this.f = f;
      g = f.CreateGraphics();
      colour = c;
      speed = s;
      size = z;
      // Create a brush of the right colour
      brush = new SolidBrush(colour);
    }

    public void Run() {
      while (!ended) {
        Draw();
        Thread.Sleep(speed); // millisecs
      }
    }

    public void Draw() {
      // Calculate a new place for a spot
      int x = r.Next(380); int y = r.Next(380);
      // Fill ellipse on screen.
      g.FillEllipse(brush, x, y, size, size);
    }
  }

  static void Main() {
    Application.Run(new SpottyThreads());
  }
}
