using System;
using System.Windows.Forms;
using System.Threading;
using System.Drawing;

class WorldTime : Form {

  ///<summary>
  /// The Clock Program          Bishop and Horspool   March 2003
  /// =================
  /// Draws a clock with moving hands
  /// Illustrates a thread for repeated drawing.
  ///</summary>

    PictureBox picture;
    Thread clockThread;
  
    WorldTime() {
      // Set up a picture box 
      picture = new PictureBox();
      picture.Size = new Size(250,250);
      this.Text = "Clock";
      // add the picture to the form
      this.Controls.Add(picture);
    
      // add the event 
      picture.Paint += new PaintEventHandler(new Clock().ClockPaint);

      // create and start a thread to control the clock
      clockThread = new Thread(new ThreadStart(this.TickClock));
      clockThread.Start();
    }

    bool ended = false;

    void TickClock() {
      while (!ended) {
        picture.Invalidate();
        Thread.Sleep(10);
      }
    }

  public class Clock {
    int seconds = 0;
    SolidBrush bluebrush, whitebrush;
    Pen blackpen;
    int start = 100;

    public Clock() {
      blackpen = new Pen(Color.Black);
      bluebrush = new SolidBrush(Color.Blue);
      whitebrush = new SolidBrush(Color.LightGray);
    }

    // The handler redraws the whole clock
	public void ClockPaint(Object source, PaintEventArgs e) {
      Graphics g = e.Graphics;
      g.DrawEllipse(blackpen,start/2,start/2,100,100);
      g.FillEllipse(whitebrush,start/2,start/2,100,100);
      g.FillEllipse(bluebrush,start,start,5,5);

      Hands(g);
      seconds++;
    }

    void Hands(Graphics g) {
      double hourAngle = 2*Math.PI*(seconds - 3*60*60)/(12*60*60);
      double minuteAngle = 2*Math.PI*(seconds - 15*60)/(60*60);
      double secondAngle = 2*Math.PI*(seconds - 15)/60;
      g.DrawLine(blackpen, start, start,
          start + (int)(30*Math.Cos(hourAngle)),
          start + (int)(30*Math.Sin(hourAngle))+5);
      g.DrawLine(blackpen,start, start, 
          start + (int)(40*Math.Cos(minuteAngle)),
          start + (int)(40*Math.Sin(minuteAngle)));
      g.DrawLine(blackpen,start, start, 
          start + (int)(45*Math.Cos(secondAngle)),
          start + (int)(45*Math.Sin(secondAngle)));
    }
  }

  static void Main() {
    Application.Run(new WorldTime());
  }
}
